/* *******************************************************************

   WebServer for the WT32-ETH01

   Hardware
     WT32-ETH01 - ESP32 Board
     Controller:   WT32-S1 (ESP32 family)
     Description:  The WT32-ETH01 is a single core version of the ESP32. 
                 

   Software
   The Software was last tested with
   - IDE 1.8.19
   - ESP32 Core 2.0.14, 2.0.6, 1.0.6    (2.0.8 can have issues)
   - ESP8266 Core 3.0.2, 2.7.4

   Short
   - Enable the Ethernet LAN 
   - simple webserver with some pages
   - Stylesheet (css) optimized for mobile devices
   - ArduinoOTA upload with Arduino IDE
   - see the full tutorial on: https://werner.rothschopf.net/microcontroller/202401_esp32_wt32_eth01_en.htm

   Open Tasks
   -

   by noiasca

   Version
   2024-01-13 initial version
***************************************************************** */

#include <Arduino.h>
#define VERSION "24.01.0"                 // the version of this sketch
#define USE_BOARD 1                       // the actual board to compile
#define TXT_BOARDNAME "ESP32WT32ETH01"    // the name of the board on webgui and Serial

#define DEBUG_ETHERNET_WEBSERVER_PORT       Serial

// Debug Level from 0 to 4
#define _ETHERNET_WEBSERVER_LOGLEVEL_       3

#include <WebServer_WT32_ETH01.h>     // WebServer_WT32_ETH01 by Khoih Hoang from Library Manager https://github.com/khoih-prog/WebServer_WT32_ETH01/

WebServer server(80);

// Optional: Select the IP address according to your local network
IPAddress myIP(172, 18, 67, 215);
IPAddress myGW(172, 18, 67, 230);
IPAddress mySN(255, 255, 255, 0);
IPAddress myDNS(8, 8, 8, 8);           // Google DNS Server IP

#include <ESPmDNS.h>                   // Bonjour/multicast DNS, finds the device on network by name   
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE

static bool eth_connected = false;

// this function is part of several examples in the internet - never seen this callback get fired
void WiFiEvent(WiFiEvent_t event) {
  switch (event) {
    case ARDUINO_EVENT_ETH_START:
      Serial.println("ETH Started");
      //set eth hostname here
      ETH.setHostname("esp32-ethernet");
      break;
    case ARDUINO_EVENT_ETH_CONNECTED:
      Serial.println("ETH Connected");
      break;
    case ARDUINO_EVENT_ETH_GOT_IP:
      Serial.print("ETH MAC: ");
      Serial.print(ETH.macAddress());
      Serial.print(", IPv4: ");
      Serial.print(ETH.localIP());
      if (ETH.fullDuplex()) {
        Serial.print(", FULL_DUPLEX");
      }
      Serial.print(", ");
      Serial.print(ETH.linkSpeed());
      Serial.println("Mbps");
      eth_connected = true;
      break;
    case ARDUINO_EVENT_ETH_DISCONNECTED:
      Serial.println("ETH Disconnected");
      eth_connected = false;
      break;
    case ARDUINO_EVENT_ETH_STOP:
      Serial.println("ETH Stopped");
      eth_connected = false;
      break;
    default:
      break;
  }
}

void setup() {
  Serial.begin(115200);
  WiFi.onEvent(WiFiEvent);
  Serial.print(F("\n" TXT_BOARDNAME "\nVersion: " VERSION " Board "));
  Serial.println(USE_BOARD);
  Serial.print  (__DATE__);
  Serial.print  (" ");
  Serial.println(__TIME__);

  // To be called before ETH.begin()
  WT32_ETH01_onEvent();
  
  //int result = ETH.begin(ETH_ADDR, ETH_POWER_PIN, ETH_MDC_PIN, ETH_MDIO_PIN, ETH_TYPE, ETH_CLK_MODE);
  int result = ETH.begin(ETH_PHY_ADDR, ETH_PHY_POWER);

  // Static IP, leave without this line to get IP via DHCP
  ETH.config(myIP, myGW, mySN, myDNS);
  WT32_ETH01_waitForConnect();  // blocks code until IP/DNS/GW was received
  
  /*
    // these functions need ESP Core 3.0.0 and are not available in 2.0.14
    while (!ETH.connected()){
    Serial.print('c');
    }
    Serial.println();
    while (!ETH.hasIP()){
    Serial.print('i');
    }
    Serial.println();
    Serial.print("autoNegotiation: "); Serial.println(ETH.autoNegotiation());
  */
  Serial.print("result: "); Serial.println(result);
  ETH.setHostname("esp32-ethernet");
  Serial.print("ETH MAC: "); Serial.println(ETH.macAddress());
  Serial.print("IPv4: "); Serial.println(ETH.localIP());
  Serial.print("fullDuplex: "); Serial.println(ETH.fullDuplex());
  Serial.print("linkSpeed: "); Serial.println(ETH.linkSpeed());

  serverSetup();
  ArduinoOTASetup();
}

void loop() {
  server.handleClient();               // call the webserver
  ArduinoOTA.handle();                 // OTA Upload via ArduinoIDE
}
//
